from AIArena.games.Connect4 import Connect4 as Connect4
from AIArena.games.GymBlackjack import GymBlackjack as GymBlackjack
from AIArena.games.GymFrozenLake import GymFrozenLake as GymFrozenLake
from AIArena.games.Arena import Arena as Arena
from AIArena.games.StockMarket import StockMarket as StockMarket
